###################################################
### chunk number 1: preliminaries
###################################################
library("zoo")


###################################################
### chunk number 2: duplicates
###################################################
## zoo series with duplicated indexes
z <- suppressWarnings(zoo(1:8, c(1, 2, 2, 2, 3, 4, 5, 5)))
z

# fix it up by averaging duplicates
aggregate(z, force, mean)

# or, fix it up by taking last in each set of duplicates
aggregate(z, force, tail, 1)

# fix it up via interpolation of duplicate times
time(z) <- na.approx(ifelse(duplicated(time(z)), NA, time(z)), na.rm = FALSE)
# if there is a run of equal times at end they
# wind up as NAs and we cannot have NA times
z[!is.na(time(z))]


###################################################
### chunk number 3: log-plot
###################################################
z <- zoo(1:100)
plot(z, log = "y", panel = function(..., log) lines(...))


###################################################
### chunk number 5: factor-1
###################################################
DF <- data.frame(time = 1:4, x = 1:4, f = factor(1:4))
zx <- zoo(DF$x, DF$time)
zf <- zoo(DF$f, DF$time)


###################################################
### chunk number 6: factor-2
###################################################
z <- zoo(data.matrix(DF[-1]), DF$time)


###################################################
### chunk number 7: lags
###################################################
z <- zoo(11:15, as.Date("2008-01-01") + c(-4, 1, 2, 3, 6))
zr <- as.zooreg(z)

lag(z)
lag(zr)

diff(log(z))
diff(log(zr))


###################################################
### chunk number 8: subtract-monthly-means
###################################################
# test data
set.seed(123)
z <- zoo(rnorm(100), as.Date("2007-01-01") + seq(0, by = 10, length = 100))

# answer
z.demean1 <- z - ave(z, as.yearmon(time(z)))


###################################################
### chunk number 9: subtract-monthly-means-2
###################################################
z.demean2 <- z - ave(z, format(time(z), "%m"))


###################################################
### chunk number 10: yearmon2
###################################################
# as.yearmon2 generic and as.yearmon2.Date method
as.yearmon2 <- function(x, ...) UseMethod("as.yearmon2")
as.yearmon2.Date <- function(x, ...) {
  y <- as.yearmon(with(as.POSIXlt(x, tz = "GMT"), 1900 + year + mon/12))
  names(y) <- x
  structure(y, class = c("yearmon2", class(y)))
}

# as.Date.yearmon2 is inverse of as.yearmon2.Date
as.Date.yearmon2 <- function(x, frac = 0, ...) {
     if (!is.null(names(x))) return(as.Date(names(x)))
     x <- unclass(x)
     year <- floor(x + .001)
     month <- floor(12 * (x - year) + 1 + .5 + .001)
     dd.start <- as.Date(paste(year, month, 1, sep = "-")) 
     dd.end <- dd.start + 32 - as.numeric(format(dd.start + 32, "%d"))
     as.Date((1-frac) * as.numeric(dd.start) + frac * as.numeric(dd.end), origin = "1970-01-01")
}

# test
dd <- seq(as.Date("2000-01-01"), length = 5, by = 32)
z <- zoo(1:5, as.yearmon2(dd))
# shows index as yearmon class
z
# recover dates to show index as Date class
aggregate(z, as.Date, force) 


###################################################
### chunk number 11: single-panel
###################################################
# custom axis for single panel plot
# label months but use larger year for Jan
# months, quarters and years should have successively larger ticks
z <- zoo(0:500, as.Date(0:500))
plot(z, xaxt = "n")
tt <- time(z)
m <- unique(as.Date(as.yearmon(tt)))
jan <- format(m, "%m") == "01"
mlab <- substr(months(m[!jan]), 1, 1)
Axis(side = 1, at = m[!jan], labels = mlab, tcl = -0.3, cex.axis = 0.7)
Axis(side = 1, at = m[jan], labels = format(m[jan], "%y"), tcl = -0.7)
Axis(side = 1, at = unique(as.Date(as.yearqtr(tt))), labels = FALSE)


###################################################
### chunk number 12: multiplesingleplot
###################################################
z3 <- cbind(z1 = z, z2 = 2*z, z3 = 3*z)
opar <- par(mfrow = c(2, 2))
tt <- time(z)
m <- unique(as.Date(as.yearmon(tt)))
jan <- format(m, "%m") == "01"
mlab <- substr(months(m[!jan]), 1, 1)
for(i in 1:ncol(z3)) {
    plot(z3[,i], xaxt = "n", ylab = colnames(z3)[i], ylim = range(z3))
    Axis(side = 1, at = m[!jan], labels = mlab, tcl = -0.3, cex.axis = 0.7)
    Axis(side = 1, at = m[jan], labels = format(m[jan], "%y"), tcl = -0.7)
    Axis(side = 1, at = unique(as.Date(as.yearqtr(tt))), labels = FALSE)
}
par(opar)


###################################################
### chunk number 13: multipanelplot
###################################################
plot(z3, screen = 1:3, xaxt = "n", nc = 2, ylim = range(z3),
  panel = function(...) {
   lines(...)
   panel.number <- parent.frame()$panel.number
   nser <- parent.frame()$nser
   # place axis on bottom panel of each column only
   if (panel.number %% 2 == 0 || panel.number == nser) { 
         tt <- list(...)[[1]]
         m <- unique(as.Date(as.yearmon(tt)))
         jan <- format(m, "%m") == "01"
         mlab <- substr(months(m[!jan]), 1, 1)
         Axis(side = 1, at = m[!jan], labels = mlab, tcl = -0.3, cex.axis = 0.7)
         Axis(side = 1, at = m[jan], labels = format(m[jan], "%y"), tcl = -0.7)
         Axis(side = 1, at = unique(as.Date(as.yearqtr(tt))), labels = FALSE)
    }
})


###################################################
### chunk number 14: plot-with-na
###################################################
z <- zoo(c(1, NA, 2, NA, 3))
plot(z) # nothing appears plotted

# try one of these

# plot points rather than lines
plot(z, type = "p") 

# omit NAs and plot that
plot(na.omit(z))

# fill in the NAs with interpolated values
plot(na.approx(z))

# plot points with lines superimposed
plot(z, type = "p")
lines(na.omit(z))


